/*
xmlrpc.js beta version 1
Tool for creating XML-RPC formatted requests in JavaScript

Copyright 2001 Scott Andrew LePera
scott@scottandrew.com
http://www.scottandrew.com/xml-rpc

License:
You are granted the right to use and/or redistribute this
code only if this license and the copyright notice are included
and you accept that no warranty of any kind is made or implied
by the author.
*/
if (!sape) var sape = {};
if (!sape.xmlrpc) sape.xmlrpc = {}

sape.xmlrpc.xml = function(method)
{
  var xml  = "<?xml version=\"1.0\"?>\n";
  xml += "<methodCall>\n";
  xml += "<methodName>" + method+ "</methodName>\n";
  xml += "<params>\n";

  for (var i = 1; i < arguments.length; i++)
  {
    var data = arguments[i];

    xml += "<param>\n";
    xml += "<value>" + sape.xmlrpc.getParamXML(sape.xmlrpc.dataTypeOf(data), data) + "</value>\n";
    xml += "</param>\n";
  }

  xml += "</params>\n";
  xml += "</methodCall>";
  return xml;
}

sape.xmlrpc.dataTypeOf = function (o)
{
  var type = typeof(o);
      type = type.toLowerCase();

  switch(type)
  {
    case "number":
      if (Math.round(o) == o) type = "int";
      else                    type = "double";
      break;
    case "object":
      var con = o.constructor;
      if (con == Date) type = "date";
      else             type = "array";
  }
  return type;
}

sape.xmlrpc.doValueXML = function(type, data)
{
  var xml = "<" + type + ">" + data + "</" + type + ">";
  return xml;
}

sape.xmlrpc.doBooleanXML = function(data)
{
  var value = (data == true)? 1:0;
  var xml = "<boolean>" + value + "</boolean>";
  return xml;
}

sape.xmlrpc.doArrayXML = function(data)
{
  var xml = "<array><data>\n";
  for (var i = 0; i < data.length; i++)
  {
    xml += "<value>" + sape.xmlrpc.getParamXML(sape.xmlrpc.dataTypeOf(data[i]), data[i]) + "</value>\n";
  }
  xml += "</data></array>\n";
  return xml;
}

sape.xmlrpc.getParamXML = function(type, data)
{
  var xml;
  switch (type)
  {
    case "array":
      xml = sape.xmlrpc.doArrayXML(data);
      break;
    case "boolean":
      xml = sape.xmlrpc.doBooleanXML(data);
      break;
    default:
      xml = sape.xmlrpc.doValueXML(type, data);
      break;
  }
  return xml;
}

sape.xmlrpc.serialize = function(mixed_val)
{
    function get_class(obj)
    {
        if (obj instanceof Object && !(obj instanceof Array) && !(obj instanceof Function) && obj.constructor)
        {
            var arr = obj.constructor.toString().match(/function\s*(\w+)/);
            if (arr && arr.length == 2)
            {
                return arr[1];
            }
        }
        return false;
    }
    switch (typeof(mixed_val))
    {
        case "string":
            return "s:" + mixed_val.length + ":\"" + mixed_val + "\";";

        case "boolean":
            return "b:" + (mixed_val ? "1" : "0") + ";";

        case "undefined":
            return "N;";

        case "number":
        case "integer":
            if (isNaN(mixed_val) || !isFinite(mixed_val))
            {
                return false;
            }
            else
            {
                return (Math.floor(mixed_val) == mixed_val ? "i" : "d") + ":" + mixed_val + ";";
            }

        case "object":
            if (mixed_val == null)
            {
                return "N;";
            }
            else if (mixed_val instanceof Array)
            {
                var idxobj = { idx: -1 };
        		var map = []
        		for(var i=0; i<mixed_val.length; i++)
                {
        			idxobj.idx++;
                    var ser = sape.xmlrpc.serialize(mixed_val[i]);
        			if (ser)
                    {
                       	map.push(sape.xmlrpc.serialize(idxobj.idx) + ser)
                    }
    		    }
                return "a:" + mixed_val.length + ":{" + map.join("") + "}"
            }
            else
            {
                var class_name = get_class(mixed_val);
                if (class_name == undefined)
                {
                    return false;
                }
                var props = new Array();
                for (var prop in mixed_val)
                {
                    var ser = sape.xmlrpc.serialize(mixed_val[prop]);
                    if (ser)
                    {
                        props.push(sape.xmlrpc.serialize(prop) + ser);
                    }
                }
                return "O:" + class_name.length + ":\"" + class_name + "\":" + props.length + ":{" + props.join("") + "}";
            }
    }
    return false;
};

sape.xmlrpc.unserialize = function(data)
{
    // http://kevin.vanzonneveld.net
    // +     original by: Arpad Ray (mailto:arpad@php.net)
    var that = this;
    var utf8Overhead = function(chr) {
        var code = chr.charCodeAt(0);
        if (code < 0x0080) {
            return 0;
        }
        if (code < 0x0800) {
             return 1;
        }
        return 2;
    };

    var read_until = function (data, offset, stopchr){
        var buf = [];
        var chr = data.slice(offset, offset + 1);
        var i = 2;
        while (chr != stopchr)
        {
            buf.push(chr);
            chr = data.slice(offset + (i - 1),offset + i);
            i += 1;
        }
        return [buf.length, buf.join('')];
    };

    var read_chrs = function (data, offset, length){
        var buf;

        buf = [];
        for (var i = 0;i < length;i++){
            var chr = data.slice(offset + (i - 1),offset + i);
            buf.push(chr);
            length -= utf8Overhead(chr);
        }
        return [buf.length, buf.join('')];
    };

    var _unserialize = function (data, offset){
        var readdata;
        var readData;
        var chrs = 0;
        var ccount;
        var stringlength;
        var keyandchrs;
        var keys;

        if (!offset) {offset = 0;}
        var dtype = (data.slice(offset, offset + 1)).toLowerCase();

        var dataoffset = offset + 2;
        var typeconvert = function(x) {return x;};

        switch (dtype){
            case 'i':
                typeconvert = function (x) {return parseInt(x, 10);};
                readData = read_until(data, dataoffset, ';');
                chrs = readData[0];
                readdata = readData[1];
                dataoffset += chrs + 1;
            break;
            case 'b':
                typeconvert = function (x) {return parseInt(x, 10) !== 0;};
                readData = read_until(data, dataoffset, ';');
                chrs = readData[0];
                readdata = readData[1];
                dataoffset += chrs + 1;
            break;
            case 'd':
                typeconvert = function (x) {return parseFloat(x);};
                readData = read_until(data, dataoffset, ';');
                chrs = readData[0];
                readdata = readData[1];
                dataoffset += chrs + 1;
            break;
            case 'n':
                readdata = null;
            break;
            case 's':
                ccount = read_until(data, dataoffset, ':');
                chrs = ccount[0];
                stringlength = ccount[1];
                dataoffset += chrs + 2;
                readData = read_chrs(data, dataoffset+1, parseInt(stringlength, 10));
                chrs = readData[0];
                readdata = readData[1];
                dataoffset += chrs + 2;
                readdata = sape.toolbar.utf8.decode(readdata);
            break;
            case 'a':
                readdata = {};
                keyandchrs = read_until(data, dataoffset, ':');
                chrs = keyandchrs[0];
                keys = keyandchrs[1];
                dataoffset += chrs + 2;

                for (var i = 0; i < parseInt(keys, 10); i++){
                    var kprops = _unserialize(data, dataoffset);
                    var kchrs = kprops[1];
                    var key = kprops[2];
                    dataoffset += kchrs;

                    var vprops = _unserialize(data, dataoffset);
                    var vchrs = vprops[1];
                    var value = vprops[2];
                    dataoffset += vchrs;

                    readdata[key] = value;
                }

                dataoffset += 1;
            break;
        }
        return [dtype, dataoffset - offset, typeconvert(readdata)];
    };
    return _unserialize((data + ''), 0)[2];
};

